"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBundleFromFolders = exports.analyzeFolders = exports.analyzeBundle = void 0;
/* eslint-disable no-await-in-loop */
const lodash_pick_1 = __importDefault(require("lodash.pick"));
const files_1 = require("./files");
const http_1 = require("./http");
const emitter_1 = __importDefault(require("./emitter"));
// import { MAX_PAYLOAD } from './constants';
const bundles_1 = require("./bundles");
const sleep = (duration) => new Promise(resolve => setTimeout(resolve, duration));
// const ANALYSIS_OPTIONS_DEFAULTS = {
//   baseURL: defaultBaseURL,
//   sessionToken: '',
//   severity: AnalysisSeverity.info,
//   symlinksEnabled: false,
//   maxPayload: MAX_PAYLOAD,
//   defaultFileIgnores: IGNORES_DEFAULT,
//   source: '',
// };
async function pollAnalysis(options) {
    let analysisResponse;
    let analysisData;
    emitter_1.default.analyseProgress({
        status: http_1.AnalysisStatus.waiting,
        progress: 0,
    });
    // eslint-disable-next-line no-constant-condition
    while (true) {
        // eslint-disable-next-line no-await-in-loop
        analysisResponse = await http_1.getAnalysis(options);
        if (analysisResponse.type === 'error') {
            return analysisResponse;
        }
        analysisData = analysisResponse.value;
        if (analysisData.status === http_1.AnalysisStatus.waiting ||
            analysisData.status === http_1.AnalysisStatus.fetching ||
            analysisData.status === http_1.AnalysisStatus.analyzing ||
            analysisData.status === http_1.AnalysisStatus.done) {
            // Report progress of fetching
            emitter_1.default.analyseProgress(analysisData);
        }
        else if (analysisData.status === http_1.AnalysisStatus.complete) {
            // Return data of analysis
            return analysisResponse;
            // deepcode ignore DuplicateIfBody: false positive it seems that interface is not taken into account
        }
        else if (analysisData.status === http_1.AnalysisStatus.failed) {
            // Report failure of analysing
            return analysisResponse;
        }
        await sleep(500);
    }
}
async function analyzeBundle(options) {
    // Call remote bundle for analysis results and emit intermediate progress
    const analysisData = await pollAnalysis(options);
    if (analysisData.type === 'error') {
        throw analysisData.error;
    }
    else if (analysisData.value.status === http_1.AnalysisStatus.failed) {
        throw new Error('Analysis has failed');
    }
    return analysisData.value;
}
exports.analyzeBundle = analyzeBundle;
async function analyzeFolders(options) {
    const remoteBundle = await createBundleFromFolders({
        ...lodash_pick_1.default(options, ['baseURL', 'sessionToken', 'source']),
        ...lodash_pick_1.default(options, ['paths', 'symlinksEnabled', 'maxPayload', 'defaultFileIgnores']),
    });
    // Analyze bundle
    if (remoteBundle === null) {
        return null;
    }
    const analysisResults = await analyzeBundle({
        ...lodash_pick_1.default(options, ['baseURL', 'sessionToken', 'source', 'severity', 'limitToFiles']),
        bundleHash: remoteBundle.bundleHash,
    });
    // TODO: expand relative file names to absolute ones
    // analysisResults.files = normalizeResultFiles(analysisData.analysisResults.files, baseDir);
    return analysisResults;
}
exports.analyzeFolders = analyzeFolders;
/**
 * Creates a remote bundle and returns response from the bundle API
 *
 * @param {CreateBundleFromFoldersOptions} options
 * @returns {Promise<RemoteBundle | null>}
 */
async function createBundleFromFolders(options) {
    const baseDir = files_1.determineBaseDir(options.paths);
    // Fetch supporte files to save network traffic
    const supportedFiles = await getSupportedFiles(options.baseURL, options.source);
    // Scan for custom ignore rules
    const fileIgnores = await files_1.collectIgnoreRules(options.paths, options.symlinksEnabled, options.defaultFileIgnores);
    emitter_1.default.scanFilesProgress(0);
    const bundleFiles = [];
    let totalFiles = 0;
    const bundleFileCollector = files_1.collectBundleFiles({
        ...lodash_pick_1.default(options, ['paths', 'symlinksEnabled', 'maxPayload']),
        baseDir,
        fileIgnores,
        supportedFiles,
    });
    for await (const f of bundleFileCollector) {
        bundleFiles.push(f);
        totalFiles += 1;
        emitter_1.default.scanFilesProgress(totalFiles);
    }
    const bundleOptions = {
        ...lodash_pick_1.default(options, ['baseURL', 'sessionToken', 'source']),
        baseDir,
        files: bundleFiles,
    };
    // Create remote bundle
    return bundleFiles.length ? bundles_1.remoteBundleFactory(bundleOptions) : null;
}
exports.createBundleFromFolders = createBundleFromFolders;
/**
 * Get supported filters and test baseURL for correctness and availability
 *
 * @param baseURL
 * @param source
 * @returns
 */
async function getSupportedFiles(baseURL, source) {
    emitter_1.default.supportedFilesLoaded(null);
    const resp = await http_1.getFilters(baseURL, source);
    if (resp.type === 'error') {
        throw resp.error;
    }
    const supportedFiles = resp.value;
    emitter_1.default.supportedFilesLoaded(supportedFiles);
    return supportedFiles;
}
//# sourceMappingURL=analysis.js.map